const { contextBridge, ipcRenderer, nativeImage } = require("electron");
const { statSync, readFileSync, existsSync } = require("fs");
const crypto = require("crypto");

let checksum = (data) => {
  return crypto.createHash("sha256").update(data).digest("hex");
};

let readFileContent = (path) => {
  if (existsSync(path) && !isFolder(path)) {
    return readFileSync(path, { encoding: "utf-8", flag: "r" });
  }
  return false;
};

let generateImagePreview = async (canvas, path) => {
  const context = canvas.getContext("2d");
  // We attempt to get the DataURL of the full image, otherwise we fallback to the thumbnail
  const base64Source =
    (await ipcRenderer.invoke("getImageDataURL", path, false)) ||
    (await ipcRenderer.invoke("getImageDataURL", path));

  const image = new Image();

  context.clearRect(0, 0, canvas.width, canvas.height);
  image.src = base64Source;

  image.onload = function () {
    canvas.width = image.width;
    canvas.height = image.height;

    const widthRatio = canvas.width / image.width;
    const heightRatio = canvas.height / image.height;
    const ratio = Math.min(widthRatio, heightRatio);
    const centerX = (canvas.width - image.width * ratio) / 2;
    const centerY = (canvas.height - image.height * ratio) / 2;

    context.clearRect(0, 0, canvas.width, canvas.height);
    context.drawImage(
      image,
      0,
      0,
      image.width,
      image.height,
      centerX,
      centerY,
      image.width * ratio,
      image.height * ratio,
    );
  };
};

let getFileInfo = (file) => {
  try {
    return statSync(file);
  } catch {
    return false;
  }
};

let getConfigDir = async () => {
  return await ipcRenderer.invoke("requestAppData");
};

let isFolder = (path) => {
  try {
    if (statSync(path).isDirectory()) {
      return true;
    } else {
      return false;
    }
  } catch {
    return false;
  }
};

contextBridge.exposeInMainWorld("api", {
  getConfigDir,
  generateImagePreview,
  readFileContent,
  getFileInfo,
  isFolder,
  checksum,
});

// Messaging bridge
contextBridge.exposeInMainWorld("messaging", {
  // Sends data from 'Renderer' to 'Main'
  // Use on 'Main': ipcMain.on(channel, (event, data) => {})
  // Use on 'Renderer': window.messaging.send(channel, data)
  send: (channel, data) => {
    ipcRenderer.send(channel, data);
  },
  // Sends data from 'Main' to 'Renderer'
  // Use on 'Main': window.webContents.send(channel, data)
  // Use on 'Renderer': window.messaging.receive(channel, (data) => {})
  receive: (channel, func) => {
    ipcRenderer.on(channel, (event, ...args) => func(...args));
  },

  invoke: (channel, path) => ipcRenderer.invoke(channel, path),
});
